<?php
/*  Copyright 2002-2003 Patrick R. Michaud (pmichaud@pobox.com)
    This file is part of PmWiki; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License
    (at your option) any later version.  See pmwiki.php for full details.

    This script enables markup of the form <<|TrailPage|>> to be 
    used to build "trails" through wiki documents.
    To enable this feature, just execute
   	include_once("scripts/trails.php");
    from local.php somewhere.
    Once enabled, the <<|TrailPage|>> markup is replaced with
    << PrevPage | TrailPage | NextPage >> on output.  TrailPage should
    contain either a bullet or number list defining the sequence of pages
    in the "trail".

    The ^|TrailPage|^ markup uses the depth of the bullets to display
    the ancestry of the TrailPage to the current one.
*/

$TrailLinkPattern = 
  "(?:($GroupNamePattern)([\\/.]))?(($WikiWordPattern)|($FreeLinkPattern))";
$DoubleBrackets["/&lt;&lt;\\|($TrailLinkPattern)\\|&gt;&gt;/e"] 
  = 'MakeTrailStop("$1");';
$DoubleBrackets["/\\^\\|($TrailLinkPattern)\\|\\^/e"] 
  = 'MakeTrailPath("$1");';

function MakeTrailStop($link) {
  global $pagename;
  $t = ReadTrail($link);
  for($i=0;$i<count($t);$i++) {
    if ($t[$i]['name']==$pagename) {
      if ($i>0) $prev = $t[$i-1]['link'];
      if ($i+1<count($t)) $next = $t[$i+1]['link'];
    }
  }
  return "&lt;&lt; $prev | $link | $next &gt;&gt;";
}

function MakeTrailPath($link) {
  global $pagename;
  $t = ReadTrail($link);
  for($i=0;$i<count($t);$i++) {
    if ($t[$i]['name']==$pagename) {
      while ($t[$i]['depth']>0) {
        $crumbs = ' | '.$t[$i]['link'].$crumbs;
        $i = $t[$i]['parent'];
      }
      return "$link$crumbs";
    }
  }
  return $link;
}
      
function ReadTrail($link) {
  global $pagename,$Newline,$TrailLinkPattern;
  $trailname = FmtWikiLink('',$link,NULL,'PageName');
  $trailpage = ReadPage($trailname);
  if ($trailpage) {
    $trailgroup = FmtPageName('$Group',$trailname);
    $n = 0;
    foreach(explode($Newline,$trailpage['text']) as $x) {
      if (preg_match("/([#*]+)\\s*($TrailLinkPattern)/",$x,$match)) {
        $t[$n]['depth'] = $depth = strlen($match[1]); 
        $t[$n]['link'] = ($match[3]>'') ? $match[2] : $trailgroup.'/'.$match[2];
        $t[$n]['name'] = FmtWikiLink('',$t[$n]['link'],NULL,'PageName',$trailpage);
        for($i=$depth;$i<10;$i++) $d[$i] = $n;
        if ($depth>1) $t[$n]['parent']=$d[$depth-1];
        $n++;
      }
    }
  }
  return $t;
}

?>
